function CRC_Table = util_crc_table_gen(CRCPolynomial, bitLen)
%Calculates table of polynomial remainders for CRC computation
%Based on example code at http://www.netrino.com/Embedded-Systems/How-To/CRC-Calculation-C-Code

if bitLen == 32
    f_bl = @(x) (uint32(x));
elseif bitLen == 8
    f_bl = @(x) (uint8(x));
else
    error('bitLen must be 8 or 32!')
end

CRC_Table = zeros(1, 256);

for n=0:255
	crc_accum = bitshift(f_bl(n),(bitLen-8));

	for m=0:7
		x = bitshift(f_bl(crc_accum), 1);
		if(crc_accum >= 2^(bitLen-1))
			crc_accum = bitxor(x, CRCPolynomial);
		else
			crc_accum = x;
		end
	end

	CRC_Table(n+1) = crc_accum;
end
