# -*- coding: utf-8 -*-
#
# mango_wlan_exp documentation build configuration file, created by
# sphinx-quickstart on Fri Apr  4 11:27:32 2014.
#
# This file is execfile()d with the current directory set to its
# containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

# os.path.abspath returns 'docs/source/' directory
import os
import sys
p = sys.path
sys.path.insert(1, os.path.abspath('../../..')) # add local wlan_exp to start of path

import wlan_exp
print('\n**********************\nImported wlan_exp from:\n  {0}'.format(wlan_exp.__file__))

import wlan_exp.version as wlan_exp_version

def gen_log_entry_types_docs(gen=True):
    RST_HDR_FILE = os.path.abspath('./log_entry_types.rst_header')
    RST_FILE_OUT = os.path.abspath('./log_entry_types.rst')

    # Check whether the entry_types.py file has been modified
    #  since the docs were last built
    import wlan_exp.log.entry_types as et
    src_mtime = os.path.getmtime(et.__file__)
    hdr_mtime = os.path.getmtime(RST_HDR_FILE)

    doc_mtime = os.path.getmtime(RST_FILE_OUT)

    if((src_mtime <= doc_mtime) and (hdr_mtime <= doc_mtime)):
        # RST file already newer than header RST and Python source
        print('\nUsing cached log entry docs')
        return

    fi = open(RST_HDR_FILE, 'r')
    fo = open(RST_FILE_OUT, 'w') # overwrite, don't append
    
    hdr = fi.read()
    fi.close()

    fo.write(hdr + "\n")
    
    from wlan_exp.log.entry_types import log_entry_types as log_entry_types
    for tid in log_entry_types.keys():
        if(type(tid) is int and tid != 0):
            fo.write(log_entry_types[tid].generate_entry_doc(fmt='rst'))

    fo.flush()
    fo.close()


# Re-generate the entry type .rst file
#  Set gen=False to disable generating entry type docs
#  In this mode the log_entry_types.rst file will still be
#   overwritten with the _header contents but will not have
#   any per-type docs
#  This mode should only be used when the entry_types.py
#   code has problems, but you still want to generate docs
gen_log_entry_types_docs(gen=True)
print('**********************\n')

#Some wlan_exp modules require numpy, but the Python environment
# on the warpproject.org server doesn't have numpy (on purpose).
# Numpy methods aren't actually called during autodoc, but a call
# to 'import numpy' must succeed for autodoc to scan every wlan_exp
# module successfully. Mock fixes this - see:
# http://read-the-docs.readthedocs.org/en/latest/faq.html#i-get-import-errors-on-libraries-that-depend-on-c-modules

# The server sets an environment variable BUILDING_DOCS_ON_SERVER when it runs post-svn-commit
#  Mock the numpy import only on the server
if os.environ.get('BUILDING_DOCS_ON_SERVER', False):
    try:
        from mock import Mock as MagicMock

        class Mock(MagicMock):
            @classmethod
            def __getattr__(cls, name):
                return Mock()

        MOCK_MODULES = ['numpy', 'pandas']
        sys.modules.update((mod_name, Mock()) for mod_name in MOCK_MODULES)
    except ImportError:
        #mock wasn't installed - keep trying, will probably fail at actual numpy import
        pass


# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.

# -- General configuration ------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.todo',
    'sphinx.ext.coverage',
    'sphinx.ext.ifconfig',
    'sphinx.ext.viewcode',
    'sphinx.ext.extlinks',
    'sphinx.ext.napoleon',
    'sphinxcontrib.fulltoc',
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The encoding of source files.
#source_encoding = 'utf-8-sig'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = u'Mango 802.11 Experiments Framework User Guide'
copyright = u'2018, Mango Communications, Inc.'
author = u'Mango Communications, Inc.'

# Napoleon settings
napoleon_use_ivar = True
napoleon_google_docstring = True
napoleon_use_param = False

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = wlan_exp_version.version_number
# The full version, including alpha/beta/rc tags.
release = wlan_exp_version.version

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#language = None

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
#today = ''
# Else, today_fmt is used as the format for a strftime call.
#today_fmt = '%B %d, %Y'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = []

# The reST default role (used for this markup: `text`) to use for all
# documents.
#default_role = None

# If true, '()' will be appended to :func: etc. cross-reference text.
add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
add_module_names = False

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# A list of ignored prefixes for module index sorting.
#modindex_common_prefix = []

# If true, keep warnings as "system message" paragraphs in the built documents.
#keep_warnings = False

autodoc_member_order = 'bysource'

# Supress warnings about .rst files not mentioned in TOC
exclude_patterns = ['globals.rst']

# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#html_theme = 'default'
#html_theme = 'nature'
#html_theme = 'classic'
#html_theme = 'alabaster'
#html_theme = 'sphinx_rtd_theme' # Methods are clearly marked, good for our method-heavy pages
#html_theme = 'bizstyle'

html_theme = 'nature'
html_css_files = ['wlan_doc.css']
html_style = 'wlan_doc.css'
html_show_sphinx = False
html_show_sourcelink = False
html_short_title = '802.11 Experiments Framework User Guide'

numfig  = True

html_sidebars = {'**': ['globaltoc.html',  'searchbox.html']}

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
#html_theme_options = {}

# Add any paths that contain custom themes here, relative to this directory.
#html_theme_path = []

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
#html_title = None

# A shorter title for the navigation bar.  Default is the same as html_title.
#html_short_title = None

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
#html_logo = None

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
#html_favicon = None

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

# Add any extra paths that contain custom files (such as robots.txt or
# .htaccess) here, relative to this directory. These files are copied
# directly to the root of the documentation.
#html_extra_path = []

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
#html_last_updated_fmt = '%b %d, %Y'

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
#html_use_smartypants = True

# Custom sidebar templates, maps document names to template names.
#html_sidebars = {}

# Additional templates that should be rendered to pages, maps page names to
# template names.
#html_additional_pages = {}

# If false, no module index is generated.
#html_domain_indices = True

# If false, no index is generated.
#html_use_index = True

# If true, the index is split into individual pages for each letter.
#html_split_index = False

# If true, links to the reST sources are added to the pages.
#html_show_sourcelink = True

# If true, "Created using Sphinx" is shown in the HTML footer. Default is True.
#html_show_sphinx = True

# If true, "(C) Copyright ..." is shown in the HTML footer. Default is True.
#html_show_copyright = True

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
#html_use_opensearch = ''

# This is the file name suffix for HTML files (e.g. ".xhtml").
#html_file_suffix = None

# Output file base name for HTML help builder.
htmlhelp_basename = 'wlan_exp_docs'

#def setup(app):
#    app.add_stylesheet("rtd_overrides.css")

# -- Options for LaTeX output ---------------------------------------------

latex_elements = {
# The paper size ('letterpaper' or 'a4paper').
'papersize': 'letterpaper',

# The font size ('10pt', '11pt' or '12pt').
'pointsize': '10pt',

# Additional stuff for the LaTeX preamble.
#'preamble': '',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
  ('index', 'mango_wlan_exp_docs.tex', u'mango_wlan_exp Documentation',
   u'Mango Communications, Inc.', 'manual'),
]

# The name of an image file (relative to this directory) to place at the top of
# the title page.
#latex_logo = None

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
#latex_use_parts = False

# If true, show page references after internal links.
#latex_show_pagerefs = False

# If true, show URL addresses after external links.
#latex_show_urls = False

# Documents to append as an appendix to all manuals.
#latex_appendices = []

# If false, no module index is generated.
#latex_domain_indices = True


# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    ('index', 'mango_wlan_exp', u'Mango 802.11 Reference Design Experiments Framework Documentation',
     [u'Mango Communications, Inc.'], 1)
]

# If true, show URL addresses after external links.
#man_show_urls = False


# -- Options for Texinfo output -------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
  ('index', 'mango_wlan_exp', u'Mango 802.11 Reference Design Experiments Framework Documentation',
   u'Mango Communications, Inc.', 'mango_wlan_exp', u'Mango 802.11 Reference Design Experiments Framework Documentation',
   'Miscellaneous'),
]

# Documents to append as an appendix to all manuals.
#texinfo_appendices = []

# If false, no module index is generated.
#texinfo_domain_indices = True

# How to display URL addresses: 'footnote', 'no', or 'inline'.
#texinfo_show_urls = 'footnote'

# If true, do not generate a @detailmenu in the "Top" node's menu.
#texinfo_no_detailmenu = False

#Check if the "doc_svn_ver.txt" file exists, written by the post-commit hook
if os.path.exists("../doc_svn_rev.txt"):
    try:
        with open("../doc_svn_rev.txt", 'r') as f:
            f_str = '-' + f.read().rstrip() #strip trailing CR-NUL
            release += f_str
    except (IOError, OSError):
        pass
